import pymannkendall as pymk


def mk(x):
    result = pymk.original_test(x)
    # p小于0.05,则可以拒绝原假设,即该数据序列存在变化趋势。
    trend = result.trend
    if trend == "increasing":
        trend = 1
    if trend == "decreasing":
        trend = 2
    if trend == "no trend":
        trend = 3

    return trend


if __name__ == "__main__":
    arr = [36.5, 36.6, 36.7, 36.1, 38, 37.2, 36.2, 36.5, 37.1, 38, 39, 41, 38.39, 37, 38, 38, 37, 36, 36]
    arr1 = []
    print(mk(arr))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2024/8/4 18:40
# @Author  : yh
# @File    : corr.py
import pandas as pd
from hdsdk.utils.feature_utils.statistics import align


def get_corr(x, y):
    """
    计算两组时间序列数据的相关系数
    :param x: 数据x
    :param y: 数据y
    :return: 相关系数
    """
    x, y, _ = align(x, y)
    data = pd.DataFrame({"数据序列A": x, "数据序列B": y})
    corrs = data.corr()
    corr = corrs["数据序列A"]["数据序列B"]
    return corr


def get_corr_without_align(x, y):
    """
    计算两组时间序列数据的相关系数
    :param x: 数据x
    :param y: 数据y
    :return: 相关系数
    """
    data = pd.DataFrame({"数据序列A": x, "数据序列B": y})
    corrs = data.corr()
    corr = corrs["数据序列A"]["数据序列B"]
    return corr


def get_corr_multi(xList, y) -> list:
    """
    计算多个时间序列数据与某时间序列的相关系数
    :param xList: 多个时间序列数据列表，array_like
    :param y: 单个时间序列数据，array_like
    :return: 相关系数列表
    """
    result = []

    if isinstance(y, dict):
        y = list(y.values())

    for x in xList:
        if isinstance(x, dict):
            x = list(x.values())
        result.append(get_corr(x, y))

    return result


def get_corr_multi_by_name(xList, nameList, y) -> dict:
    """
    计算多个时间序列数据与某时间序列的相关系数，以hashmap形式返回，带测点名称
    :param xList: 多个时间序列数据列表，array_like
    :param nameList: 多个时间序列数据对应测点名称列表，array_like
    :param y: 单个时间序列数据，array_like
    :return:
    """
    result = {}

    if isinstance(y, dict):
        y = list(y.values())

    for x, name in zip(xList, nameList):
        if isinstance(x, dict):
            x = list(x.values())
        result[name] = get_corr(x, y)

    return result


if __name__ == '__main__':
    x = [1, 2, 3, 4, 7, ]
    y = [2, 3, 4.1, 5, 8]
    # data = np.array([x,y]).transpose()
    # data1 = np.array([x,y])
    # # data = np.concatenate((np.array(x),np.array(y)))
    # print(data)
    # df = pd.DataFrame(data,index= ["001","002","003","004"],columns=["a","b"]).transpose()
    # print(df)
    c = get_corr(x, y)

    print(c)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2024/8/4 16:54
# @Author  : yh
# @File    : constants.py

# ----------------conf-----------------
VAR_STAT_FEATURE_FORMAT = {
    "mean": None,
    "std": None,
    "maximum": None,
    "minimum": None,
    "corr_pw": None,
    "corr_wh": None,
    "slope": None,
    "stability_factor": None,
}
WORKING_CONDITION_VAR_STAT_FEATURE_FORMAT = {
    "pw": {
        "mean": None,
        "std": None,
        "maximum": None,
        "minimum": None,
        "slope": None,
        "tendency": None,
    },
    "wh": {
        "mean": None,
        "std": None,
        "maximum": None,
        "minimum": None,
        "slope": None,
        "tendency": None,
    },
    "dykd": {
        "mean": None,
        "std": None,
        "maximum": None,
        "minimum": None,
        "slope": None,
        "tendency": None,
    }
}
# ----------------DTW-----------------
# 计算了多少班次
DTW_WARP = 1
# 在路径的非对称移动上应用的权重，随着s变大，扭曲路径越来越偏向对角线
DTW_S = 1.0




from hdsdk.conf.common_config import UNPROCESSABLE_ENTITY


class GroupNumError(Exception):
    def __init__(self, value, diagnosis_result):
        self.value = value
        self.message = f"获取不到判据绑定测点组。"
        self.code = UNPROCESSABLE_ENTITY
        self.result = diagnosis_result
        super().__init__(self.message)


class GroupDataError(Exception):
    def __init__(self, value, diagnosis_result):
        self.value = value
        self.code = UNPROCESSABLE_ENTITY
        self.result = diagnosis_result
        self.message = f"测点组历史数据不存在。"
        super().__init__(self.message)


class PowerDataError(Exception):
    def __init__(self, value, diagnosis_result):
        self.value = value
        self.code = UNPROCESSABLE_ENTITY
        self.result = diagnosis_result
        self.message = f"有功功率历史数据不存在。"
        super().__init__(self.message)


# 这是机组启动判断
class StableOperationError(Exception):
    def __init__(self, value, diagnosis_result):
        self.value = value
        self.code = UNPROCESSABLE_ENTITY
        self.result = diagnosis_result
        self.message = f"机组不在启机阶段，不进行数据分析"
        super().__init__(self.message)


class ThresholdDataError(Exception):
    def __init__(self, value, diagnosis_result):
        self.value = value
        self.code = UNPROCESSABLE_ENTITY
        self.result = diagnosis_result
        self.message = f"测点组预警值配置不存在。"
        super().__init__(self.message)

class ThresholdError(Exception):
    def __init__(self, value, diagnosis_result):
        self.value = value
        self.code = UNPROCESSABLE_ENTITY
        self.result = diagnosis_result
        self.message = f"动态阈值条数与测点条数不匹配。"
        super().__init__(self.message)


class ThresholdZeroError(Exception):
    def __init__(self, value, diagnosis_result):
        self.value = value
        self.code = UNPROCESSABLE_ENTITY
        self.result = diagnosis_result
        self.message = f"动态阈值限值为0，无效阈值。"
        super().__init__(self.message)


class PointsError(Exception):
    def __init__(self, value, diagnosis_result):
        self.value = value
        self.message = f"获取不到判据绑定测点数据。"
        self.code = UNPROCESSABLE_ENTITY
        self.result = diagnosis_result
        super().__init__(self.message)

class ParamZeroError(Exception):
    def __init__(self, value, diagnosis_result):
        self.value = value
        self.code = UNPROCESSABLE_ENTITY
        self.result = diagnosis_result
        self.message = f"判据参数值为空"
        super().__init__(self.message)


class JudgeConfigError(Exception):
    def __init__(self, value, diagnosis_result):
        self.value = value
        self.code = UNPROCESSABLE_ENTITY
        self.result = diagnosis_result
        self.message = f"故障判据配置不存在"
        super().__init__(self.message)


from math import isinf
import numpy as np
from numpy import array, zeros, full, argmin, inf
from hdsdk.utils.feature_utils.constants import DTW_WARP, DTW_S


def dtw(x, y, dist=None, wrap=DTW_WARP, w=inf, s=DTW_S):
    '''
        x: N1*M array
        y: N2*M array
        dist: 用作成本度量的距离
        wrap: 计算了多少班次
        w: 窗口大小限制匹配条目索引之间的最大值
        s: 在路径的非对称移动上应用的权重，随着s变大，扭曲路径越来越偏向对角线
        返回最小距离、成本矩阵、累积成本矩阵和包裹路径
    '''

    manhattan_distance = lambda x, y: np.abs(x - y)

    if dist is None:
        dist = manhattan_distance

    r, c = len(x), len(y)
    if not isinf(w):
        D0 = full((r + 1, c + 1), inf)
        for i in range(1, r + 1):
            D0[i, max(1, i - w):min(c + 1, i + w + 1)] = 0
        D0[0, 0] = 0
        # print(D0, "D0-1")
    else:
        D0 = zeros((r + 1, c + 1))
        D0[0, 1:] = inf
        D0[1:, 0] = inf
        # print(D0, "D0-1 初始化累积距离矩阵")
    D1 = D0[1:, 1:]
    # print(D1, "D1-1")
    for i in range(r):
        for j in range(c):
            if (isinf(w) or (max(0, i - w) <= j <= min(c, i + w))):
                D1[i, j] = dist(x[i], y[j])
    # print(D1, "D1-2 初始化距离矩阵")
    C = D1.copy()
    # print(C, 'C')
    jrange = range(c)
    # print(jrange, 'jrange-0 列数')
    # print(D0, 'D0-2 赋值距离后但还未累计的累计距离矩阵')
    for i in range(r):  # i 行数
        if not isinf(w):
            jrange = range(max(0, i - w), min(c, i + w + 1))
        for j in jrange:
            min_list = [D0[i, j]]
            for k in range(1, wrap + 1):
                i_k = min(i + k, r)
                j_k = min(j + k, c)
                min_list += [D0[i_k, j] * s, D0[i, j_k] * s]
                # print(min_list, "min_list-k")
            D1[i, j] += min(min_list)
    # print(D1, "D1-3 累积距离矩阵")
    if len(x) == 1:
        path = zeros(len(y)), range(len(y))
    elif len(y) == 1:
        path = range(len(x)), zeros(len(x))
    else:
        path = _traceback(D0)
    return D1[-1, -1], C, D1, path


# 求相关路径
def _traceback(D):
    i, j = array(D.shape) - 2
    p, q = [i], [j]
    # print(D)
    # print(i,j,'i,j')
    # print(p,q,'p,q')
    while (i > 0) or (j > 0):
        tb = argmin((D[i, j], D[i, j + 1], D[i + 1, j]))
        if tb == 0:
            i -= 1
            j -= 1
        elif tb == 1:
            i -= 1
        else:
            j -= 1
        p.insert(0, i)
        q.insert(0, j)
    # print(array(p), "array(p)")
    # print(array(q), "array(q)")
    return array(p), array(q)


if __name__ == "__main__":
    x = np.array([2, 0, 1, 1]).reshape(-1, 1)
    y = np.array([1, 1, 2]).reshape(-1, 1)
    manhattan_distance = lambda x, y: np.abs(x - y)
    d, cost_matrix, acc_cost_matrix, path = dtw(x, y)
    print(d)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2024/8/27 10:51
# @Author  : yh
# @File    : timestamp2utc.py
from datetime import datetime, timedelta


def timestamp2utc(ms_timestamp):
    # 毫秒级时间戳转换为秒
    timestamp_in_seconds = ms_timestamp / 1000.0
    utc_time = datetime.fromtimestamp(timestamp_in_seconds)
    return utc_time.strftime("%Y-%m-%d %H:%M:%S")


if __name__ == '__main__':
    # 示例使用
    ms_timestamp = 1615112345321  # 假设这是一个毫秒级时间戳
    utc_time = timestamp2utc(ms_timestamp)
    print(utc_time)  # 输出转换后的UTC时间



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2024/8/6 16:24
# @Author  : yh
# @File    : point_config.py
import os

# 获取环境变量
# env = os.getenv('ENV', 'production')
CLIENT_URL = os.getenv('CLIENT_URL', '')

# 根据环境选择 URL
if CLIENT_URL == "":
    BACKEND_CLIENT_URL = "http://ssms-backend:8080"
    EXT_API_CLIENT_URL = "http://light-extendapi:8080"
else:
    BACKEND_CLIENT_URL = CLIENT_URL + '/backend-api'
    EXT_API_CLIENT_URL = CLIENT_URL + "/ext-api"

# api_client配置
BACKEND_API_PATH = '/api/v1/sso/login'
EXT_API_BASE_PATH = '/api/v1/meta_api/call'

BASE_DIAGNOSIS_URL = 'http://localhost:port/api/v1/algorithm/val'

CLIENT_USERNAME = "Admin"
CLIENT_PASSWORD = "Admin@123"


def update_client_url(url):
    global BACKEND_CLIENT_URL, EXT_API_CLIENT_URL
    BACKEND_CLIENT_URL = url + "/backend-api"
    EXT_API_CLIENT_URL = url + "/ext-api"




import json
import os
from hdsdk.conf.url_config import update_client_url
from hdsdk.utils.data_util import get_judge_config
from hdsdk.utils.data_util import api, get_history_data_by_group, get_threshold_config, get_pw_point
from hdsdk.utils.feature_utils.statistics import get_mean
from hdsdk.conf.common_config import DNY_HIGH, DNY_LOW, ALERT_LOW_1, ALERT_LOW_2, ALERT_HIGH_1, \
    ALERT_HIGH_2, PW_STABILITY_FACTOR, PW_STABILITY_LIMIT_VALUE, DEFAULT_FALSE_PROP
from hdsdk.exceptions import GroupNumError, GroupDataError, PowerDataError, StableOperationError, \
    ThresholdError, ThresholdDataError, ThresholdZeroError, PointsError, ParamZeroError, JudgeConfigError


class AlgorithmUtils:
    def __init__(self, request_data=None, client_url=None):
        self.request_data = None
        self.judge_id = None
        self.start = 0
        self.end = 0
        self.algorithm_fitting = 0
        self.algorithms = []
        self.desc = ''
        self.groups = []
        self.name = ""
        self.param = []
        self.probability = 0.0
        self.judge_type = ""
        self.unit_num = ""
        self.threshold = {}
        self.data = {}
        self.working_power = []
        self.work_data = {}
        #
        self.diagnosis_result = None
        self.pw_data = None
        self.points = []
        self.groups_data = {}
        self.group_data = {}
        self.threshold_dict = {}

        # 如果初始数据存在，调用 updateData 进行初始化
        if request_data:
            self.update_data(request_data)

        if client_url:
            update_client_url(client_url)
            api.api_client.login()


    def update_data(self, request_data):
        self.request_data = request_data
        self.judge_id = request_data.get('judge_id')
        self.start = int(request_data.get('start', 0))
        self.end = int(request_data.get('end', 0))

        debugMode = os.getenv('DEBUG_MODE', '')
        # 调试模式
        if debugMode == '1':
            judge_config = get_judge_config(self.judge_id)
            # 检查配置是否存在
            self.check_judge_config_data(judge_config)
            # 处理json数据
            judge_config["param"] = json.loads(judge_config.get('param', []))
            judge_config["groups"] = json.loads(judge_config.get('groups', []))
            judge_config["algorithms"] = json.loads(judge_config.get('algorithms', []))
            # 赋值给request_data
            self.request_data = judge_config

        self.param = self.request_data.get('param', [])
        self.groups = self.request_data.get('groups', [])
        self.algorithms = self.request_data.get('algorithms', [])

        self.algorithm_fitting = int(self.request_data.get('algorithm_fitting', 0))
        self.desc = self.request_data.get('desc', '')
        self.name = self.request_data.get('name', "")
        self.probability = self.request_data.get('probability', 0.0)
        self.judge_type = self.request_data.get('type', "")
        self.unit_num = self.request_data.get("unitNum", "")
        self.threshold = self.request_data.get('threshold', {})

        # 接口获取
        self.data = self.request_data.get('data', {})
        self.working_power = self.request_data.get("working_power", [])
        self.work_data = self.request_data.get("work_data", {})
        #
        # 更新输出结果
        self.diagnosis_result = {
            "judge_id": self.judge_id,
            "value": self.get_default_value()
        }

    def get_default_value(self):
        if self.judge_type == 'C':
            return False
        else:  # 默认 P 类型
            return DEFAULT_FALSE_PROP

    # 数据预处理
    def dataPreprocessing(self, request_data=None):
        if request_data:
            self.update_data(request_data)

        # 检测绑定测点组是否为空
        self.check_group_number(self.groups)

        # 如果传入 data 为空，读取数据库设备测点数据
        if not self.data:
            groups_result = get_history_data_by_group(self.unit_num, self.start, self.end, self.groups)
            if groups_result.get('code') == 200:
                self.groups_data = groups_result.get('data', {}).get('points', [])
        else:
            self.groups_data = self.data

        # 检测组历史数据是否为空
        self.check_group_data(self.groups_data)
        print("测点数据：", self.groups_data)

        # 如果传入 working_power、work_data为空，读取数据库有功功率测点数据
        if not self.working_power:
            pw_point = get_pw_point()
            self.working_power =  [pw_point[0]['val']] if pw_point else []

        if not self.work_data or self.work_data.get('code') != 200:
            # 查询15min内有功功率测点数据
            self.work_data = get_history_data_by_group(self.unit_num, self.start, self.end, self.working_power)

        if self.work_data.get('code') == 200:
            data_points = self.work_data.get('data', {}).get('points', {})
            self.pw_data = data_points.get(self.working_power[0])

        # 检测有功功率测点数据是否为空
        self.check_power_data(self.pw_data)

    # 稳定性分析
    def stabilityAnalysis(self):
        pw_list = []
        for value in self.pw_data.values():
            pw_list = list(value.values())
        matched_pw_list = list(filter(lambda x: x > PW_STABILITY_LIMIT_VALUE, pw_list))

        # 检测机组稳定性
        self.check_stable_operation_data(matched_pw_list, pw_list)

    def preCriterion(self):
        # 单个测点组，取测点组名称
        group_name = self.groups[0]
        # 获取测点组数据
        self.group_data = self.groups_data.get(group_name)
        # 获取测点组包含的全部测点列表
        self.points = list(self.group_data.keys())

        self.threshold_dict = self.threshold.get(group_name, {})

        if not self.threshold_dict:
            # 查询预警值
            thresholds = get_threshold_config(self.unit_num, group_name)
            # 检查是否为空
            self.check_threshold_data(thresholds)

            # 预警值处理
            for point in self.points:
                # 查找匹配的阈值
                threshold = next((threshold for threshold in thresholds if threshold.get('points_id') == point), None)
                if threshold:
                    self.threshold_dict[point] = {
                        DNY_HIGH: threshold.get(DNY_HIGH),
                        DNY_LOW: threshold.get(DNY_LOW),
                        ALERT_HIGH_1: threshold.get(ALERT_HIGH_1),
                        ALERT_HIGH_2: threshold.get(ALERT_HIGH_2),
                        ALERT_LOW_1: threshold.get(ALERT_LOW_1),
                        ALERT_LOW_2: threshold.get(ALERT_LOW_2)
                    }

        # 检测动态阈值条数与测点条数是否匹配
        self.check_threshold_number_data(self.threshold_dict, self.points)

        # 检查动态阈值是否为0，动态阈值为0表示未进行赋值
        for dny_value in self.threshold_dict.values():
            self.check_threshold_zero_data(dny_value)

    def criterion_avg_dny_high(self):
        self.preCriterion()

        self.check_param_zero_data(self.param)

        algorithm_param = self.param[0]

        flag = False
        for point in self.points:
            datadict = self.group_data.get(point)
            # 检查数据是否存在
            self.check_points_data(datadict)

            datalist = list(datadict.values())
            avg = get_mean(datalist)
            dny_high = self.threshold_dict.get(point).get(DNY_HIGH)
            if avg > algorithm_param * dny_high:
                flag = True
                break

        self.diagnosis_result.update({"value": flag})

        return self.diagnosis_result

    def criterion_prob_dny_high(self):
        self.preCriterion()

        self.check_param_zero_data(self.param)

        algorithm_param = self.param[0]

        probs = []
        for point in self.points:
            datadict = self.group_data.get(point)
            # 检查数据是否存在
            self.check_points_data(datadict)

            datalist = list(datadict.values())
            limit = self.threshold_dict.get(point).get(DNY_HIGH)
            matched_datalist = list(filter(lambda x: x > algorithm_param * limit, datalist))
            prob = len(matched_datalist) / len(datalist)
            probs.append(prob)

        final_prob = max(probs)

        self.diagnosis_result.update({"value": final_prob})

        return self.diagnosis_result

    def criterion_prob_dny_low(self):
        self.preCriterion()

        self.check_param_zero_data(self.param)

        algorithm_param = self.param[0]

        probs = []
        for point in self.points:
            datadict = self.group_data.get(point)
            # 检查数据是否存在
            self.check_points_data(datadict)

            datalist = list(datadict.values())
            limit = self.threshold_dict.get(point).get(DNY_LOW)
            matched_datalist = list(filter(lambda x: x < algorithm_param * limit, datalist))
            prob = len(matched_datalist) / len(datalist)
            probs.append(prob)

        final_prob = max(probs)

        self.diagnosis_result.update({"value": final_prob})

        return self.diagnosis_result

    def criterion_prob_nonzero(self):
        self.preCriterion()

        probs = []
        for point in self.points:
            datadict = self.group_data.get(point)
            # 检查数据是否存在
            self.check_points_data(datadict)

            datalist = list(datadict.values())
            matched_datalist = list(filter(lambda x: x != 0, datalist))
            prob = len(matched_datalist) / len(datalist)
            probs.append(prob)

        final_prob = max(probs)

        self.diagnosis_result.update({"value": final_prob})

        return self.diagnosis_result

    def criterion_prob_above_min(self):
        self.preCriterion()

        self.check_param_zero_data(self.param)

        algorithm_param = self.param[0]

        probs = []
        for point in self.points:
            datadict = self.group_data.get(point)
            # 检查数据是否存在
            self.check_points_data(datadict)

            datalist = list(datadict.values())
            minimum = min(datalist)
            matched_datalist = list(filter(lambda x: x - minimum > algorithm_param, datalist))
            prob = len(matched_datalist) / len(datalist)
            probs.append(prob)

        final_prob = max(probs)

        self.diagnosis_result.update({"value": final_prob})

        return self.diagnosis_result

    def criterion_prob_alert_high(self):
        self.preCriterion()

        probs = []
        for point in self.points:
            datadict = self.group_data.get(point)
            # 检查数据是否存在
            self.check_points_data(datadict)

            datalist = list(datadict.values())
            limit = self.threshold_dict.get(point).get(ALERT_HIGH_1)
            matched_datalist = list(filter(lambda x: x >= limit, datalist))
            prob = len(matched_datalist) / len(datalist)
            probs.append(prob)

        final_prob = max(probs)

        self.diagnosis_result.update({"value": final_prob})

        return self.diagnosis_result

    def criterion_check_empty(self):
        self.preCriterion()

        flag = self.get_default_value()

        for point in self.points:
            if self.group_data.get(point) is None or len(self.group_data.get(point)) == 0:
                flag = 1

        self.diagnosis_result.update({"value": flag })

        return self.diagnosis_result

    def criterion_custom(self, calc_method):
        self.preCriterion()

        final_prob = calc_method(self)

        self.diagnosis_result.update({"value": final_prob })

        return self.diagnosis_result


    # 检查测点组是否为空
    def check_group_number(self, groups):
        if groups is None or len(groups) == 0:
            raise GroupNumError(groups, self.diagnosis_result)
        return groups

    def check_group_data(self, groups_data):
        if groups_data is None or len(groups_data) == 0:
            raise GroupDataError(groups_data, self.diagnosis_result)
        return groups_data

    def check_stable_operation_data(self, matched_pw_list, pw_list):
        if len(matched_pw_list) / len(pw_list) < PW_STABILITY_FACTOR:
            raise StableOperationError(len(matched_pw_list) / len(pw_list), self.diagnosis_result)
        return len(matched_pw_list) / len(pw_list)

    # 测点组预警值配置不存在
    def check_threshold_data(self, thresholds):
        if thresholds is None or len(thresholds) == 0:
            return ThresholdDataError(thresholds, self.diagnosis_result)
        return thresholds

    # 检测动态阈值条数与测点条数是否匹配
    def check_threshold_number_data(self, threshold, points):
        if len(threshold) != len(points):
            raise ThresholdError([len(threshold), len(points)], self.diagnosis_result)
        return [len(threshold), len(points)]

    # 检查动态阈值是否为0，动态阈值为0表示未进行赋值
    def check_threshold_zero_data(self, dny_value):
        if dny_value.get(DNY_HIGH, 0) == 0 or dny_value.get(DNY_LOW, 0) == 0:
            raise ThresholdZeroError(dny_value, self.diagnosis_result)
        return dny_value

    def check_points_data(self, points_data):
        if points_data is None:
            raise PointsError(points_data, self.diagnosis_result)
        return points_data

    def check_power_data(self, pw_data):
        if pw_data is None or len(pw_data) == 0 or any(value is None for value in pw_data.values()):
            raise PowerDataError(pw_data, self.diagnosis_result)
        return pw_data

    def check_param_zero_data(self, param):
        if param is None or len(param) == 0:
            raise ParamZeroError(param, self.diagnosis_result)
        return param

    def check_judge_config_data(self, config):
        if config is None:
            raise JudgeConfigError(config, self.diagnosis_result)
        return config


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2024/10/22 16:00
# @Author  : yh
# @File    : api.py
from hdsdk.client.api_client import APIClient
import time


class Api(object):
    def __init__(self):
        self.api_client = APIClient()

    def query_realtime_val(self, param):
        """
        查询实时测点数据
        :param param:
        :return:
        """
        res = self.api_client.getExtApi(url='/data/real/val', param=param)
        if res['code'] == 200:
            return res['data']
        return {}

    def query_history_val(self, param):
        """
        查询实时测点数据
        :param param:
        :return:
        """
        res = self.api_client.getExtApi(url='/data/his/val', param=param)
        # if res['code'] == 200:
        #     return res['data']
        return res

    '/data/his/group/code'

    def query_history_val_by_group(self, param):
        """
        按测点组查询历史数据
        :param param:
        :return:
        """
        res = self.api_client.getExtApi(url='/data/his/group/code', param=param)
        # if res['code'] == 200:
        #     return res['data']
        return res

    def query_point_by_tag(self, param):
        """
        按标签查询测点集合
        :param param:
        :return:
        """
        # param = {
        #     'funName': 'vib_property',
        #     'unitNum': 1,
        #     'state': 1
        # }
        res = self.api_client.getExtApi(url='/global/function/points', param=param)
        point_set = []
        for item in res['data'].values():
            for point in item.keys():
                point_set.append(point)
        return point_set

    def save_inc_features(self, param):
        """
        写入数据特征增量表
        :param param:
        :return:
        """
        res = self.api_client.getExtApi(url='/inc/property/data/write', param=param)
        return res

    def save_inc_vib_relativity(self, param):
        """
        写入振动相关性特征增量表
        :param param:
        :return:
        """
        res = self.api_client.getExtApi(url='/inc/vib/relativity/write', param=param)
        return res

    def save_inc_working_condition_analysis(self, param):
        """
        写入工况数据特征表增量表
        :param param:
        :return:
        """
        res = self.api_client.getExtApi(url='/inc/working/data/query', param=param)
        return res

    def query_inc_features(self, param):
        """
        查询数据特征增量表
        :param param:
        :return:
        """
        res = self.api_client.getExtApi(url='/inc/property/data/query/point', param=param)
        return res

    def query_alert_point(self, param):
        """
        获取报警/异常功能所需测点
        :param param:
        :return:
        """
        res = self.api_client.getExtApi(url='/global/warning/points', param=param)
        keys = []
        for key in res['data'].keys():
            keys.append(key)
        return keys

    def save_inc_alert_event(self, param):
        """
        写入报警/异常事件
        :param param:
        :return:
        """
        res = self.api_client.getExtApi(url='/inc/abnormity/write/event', param=param)
        return res

    def save_inc_alert_span(self, param):
        """
        写入报警时间
        :param param:
        :return:
        """
        res = self.api_client.getExtApi(url='/inc/abnormity/write/record', param=param)
        return res

    def sdk_call(self, param):
        """
        调用sdk
        :param param:
        :return:
        """
        res = self.api_client.getExtApi(url='/sdk/call', param=param)
        return res


if __name__ == '__main__':
    api = Api()
    res = Api().query_history_val(
        {
            'unitNum': 1,
            'start': int((time.time() - 600) * 1000),
            'end': int(time.time() * 1000),
            'body': [
                {
                    'code': 'aco_cow_liq_2'
                },
                {
                    'code': 'ace_pow_1'
                }
            ]
        }
    )
    print(res)
    # 测点组查历史数据
    group = [
        {
            'group': 'upg_cow_liq'
        },
        {
            'group': 'ace_pow'
        },
    ]
    param = {
        'unitNum': 1,
        'start': int((time.time() - 600) * 1000),
        'end': int(time.time() * 1000),
        'body': group
    }
    history_data = api.query_history_val_by_group(param)
    print(history_data)



DEFAULT_FALSE_PROP = 0
UNPROCESSABLE_ENTITY = 422

DNY_HIGH = 'dny_high'
DNY_LOW = 'dny_low'
ALERT_HIGH_1 = 'alert_high_1'
ALERT_HIGH_2 = 'alert_high_2'
ALERT_LOW_1 = 'alert_low_1'
ALERT_LOW_2 = 'alert_low_2'

PW_STABILITY_LIMIT_VALUE = 10
PW_STABILITY_FACTOR = 0.0

HISTORY_DATA_INTERVAL = 10 * 60
TIME_WINDOW_INTERVAL = 20

INCREASE_TENDENCY_FACTOR = 0.9
DECREASE_TENDENCY_FACTOR = 0.9




from hdsdk.client.api import Api
from hdsdk.utils.base_util import convert_time_to_unit

api = Api()


# 通过测点获取设备测点历史数据
def get_history_data_by_point(unit_num, start, end, codes):
    return api.query_history_val(
        {
            'unitNum': unit_num,
            'start': int(start * 1000),
            'end': int(end * 1000),
            'body': [{"code": item} for item in codes]
        }
    )


# 通过测点组获取设备测点历史数据
def get_history_data_by_group(unit_num, start, end, groups):
    return api.query_history_val_by_group(
        {
            'unitNum': unit_num,
            'start': int(start * 1000),
            'end': int(end * 1000),
            'body': [{"group": item} for item in groups]
        }
    )


# 获取判据配置
def get_judge_config(judge_id):
    param = {
        "method": "db.exec",
        "args": [f"select * from meta_rt_fault_judge_config where del_flag = 0 and judge_id = '{judge_id}'"]
    }
    res = api.sdk_call(param)
    if len(res) > 0:
        return res[0]
    return None


# 获取数据特征信息表
def get_device_data_information(code, start, end):
    # 增量表查询时间要转换纳秒
    start_time = convert_time_to_unit(start)
    end_time = convert_time_to_unit(end)
    param = {
        "method": "influxdb.queryRaw",
        "args": [
            f"select * from device_data_Information where code = '{code}' and time >= {start_time} and time <= {end_time}"]
    }
    res = api.sdk_call(param)
    return res


# 获取判据配置
def get_pw_point():
    param = {
        "method": "db.exec",
        "args": [
            f"select * from meta_rt_point_value where del_flag = 0 and meta_rt_point_value.point_id = 'Workingpower'"]
    }
    res = api.sdk_call(param)

    return res


# 获取阈值配置
def get_threshold_config(unit_num, group_id):
    param = {
        "method": "db.select",
        "args": ["meta_rt_threshold_config", {
            "unitNum": unit_num,
            "group_id": group_id,
            "del_flag": 0
        }]
    }
    return api.sdk_call(param)


def result(code=200, data={}, msg=""):
    res = {"code": code, "data": data, "msg": msg}
    print("诊断结果：", res)
    return res


if __name__ == '__main__':
    print(get_pw_point()[0]['val'])
    a = get_pw_point()
    threshold_config = get_threshold_config('1', 'upg_cow_wtp')
    print(threshold_config)
    # judge = get_judge_config('V01501')
    # print(judge)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2024/10/15 13:17
# @Author  : yh
# @File    : api_client.py
import requests
import hdsdk.conf.url_config as url_config


class APIClient:
    requests_urls = {
        # 按测点查询实时数据
        '/data/real/val': 'cbba1bee-1856-49ec-be41-6016bd88da30',
        # 按测点查询历史数据
        '/data/his/val': '7326296b-b5e9-4656-aa4c-f1d1f4fa4e19',
        # 按测点组查询历史数据
        '/data/his/group/code': 'f9d1211d-c8b1-4a17-ae6e-87d5784fe430',
        # 根据功能标签获取测点
        '/global/function/points': '48a3908d-cc05-42f9-b813-9f850db555c4',
        # 写入数据特征表
        '/inc/property/data/write': 'f38e27e4-2b5b-4ce0-924f-738989e1252a',
        # 写入振动相关性特征
        '/inc/vib/relativity/write': 'cc78ae90-8bb4-4e7e-916f-662e31fcbd5b',
        # 写入工况数据特征表
        '/inc/working/data/query': '95da39a8-ee9f-4c06-9cef-6c739da5f40a',
        # 查询数据特征表数据
        '/inc/property/data/query/point': '8c70d305-fff1-4ad1-8e8f-1c45f8778a93',
        # 获取报警/异常功能所需测点
        '/global/warning/points': 'a5c8df4a-9135-4d41-837a-5a028f26433a',
        # 写入报警/异常事件
        '/inc/abnormity/write/event': '1e41ddce-b786-4556-92ac-136b40be84b7',
        # 写入报警时间
        '/inc/abnormity/write/record': '288ca54f-6ecb-492c-a724-7260a02a72aa',
        # 写入报警时间
        '/sdk/call': 'b43b8d51-eec1-4647-a2a1-fe3096f41ac5',

    }

    def __init__(self):
        self.org_domain = 'cvxa3663'
        self.token = ''
        self.login()

    def login(self):
        endpoint = url_config.BACKEND_CLIENT_URL + url_config.BACKEND_API_PATH
        data = {
            'account': url_config.CLIENT_USERNAME,
            'password': url_config.CLIENT_PASSWORD
        }
        res = self.post(endpoint, data=data)
        if res and res['code'] == 200 and res['data']:
            self.token = res['data']['token']

    def getExtApi(self, id='', url='', param=None):
        if param is None:
            param = {}
        endpoint = url_config.EXT_API_CLIENT_URL + url_config.EXT_API_BASE_PATH
        if id == '':
            id = self.requests_urls.get(url)
        data = {'id': id, 'param': param}
        res = self.post(endpoint, data=data)
        if res['code'] == 200 and res['data']:
            return res['data']
        return res

    def post(self, endpoint, data=None, headers=None):
        if headers == None:
            headers = {'token': self.token, 'org_domain': self.org_domain}
        response = requests.post(endpoint, json=data, headers=headers)
        if response.status_code == 200:
            return response.json()
        return response


if __name__ == '__main__':
    api_client = APIClient()

    param = {
        'funName': 'vib_property',
        'unitNum': 1,
        'state': 1
    }
    res = api_client.getExtApi(url='/global/function/points', param=param)
    point_set = []
    for item in res['data'].values():
        for point in item.keys():
            point_set.append(point)
    print(res)
    print(point_set)



from datetime import datetime
import pytz
import json


def parse_json(json_string, default_value=None):
    try:
        return json.loads(json_string)
    except (json.JSONDecodeError, TypeError) as e:
        print(f"Failed to parse JSON: {e}")
        return default_value


def convert_time_to_unit(time_value, target_unit='ns'):
    if isinstance(time_value, str):
        time_value = int(time_value.strip())
    elif not isinstance(time_value, (int, float)):
        raise time_value

    time_str = str(time_value)
    length = len(time_str)

    if length > 19:
        time_str = time_str[:19]
        length = 19

    if length < 19:
        time_str = time_str + '0' * (19 - length)

    time_in_ns = int(time_str)

    if target_unit == 's':  # 转换为秒
        return time_in_ns / 1_000_000_000
    elif target_unit == 'ms':  # 转换为毫秒
        return time_in_ns / 1_000_000
    elif target_unit == 'ns':  # 保持为纳秒
        return time_in_ns
    else:
        raise time_value


def convert_to_china_time(date_string):
    # 定义 UTC 时区和中国时区
    utc_timezone = pytz.utc
    china_timezone = pytz.timezone('Asia/Shanghai')

    # 将日期时间字符串转换为 datetime 对象，并设置为 UTC 时区
    dt_utc = datetime.strptime(date_string, '%Y-%m-%dT%H:%M:%SZ')
    dt_utc = utc_timezone.localize(dt_utc)

    # 转换为中国时区
    dt_china = dt_utc.astimezone(china_timezone)

    # 格式化为指定格式的字符串并返回
    china_time_string = dt_china.strftime('%Y-%m-%d %H:%M:%S')
    return china_time_string


def convert_to_china_timestamp(date_string):
    # 定义 UTC 时区和中国时区
    utc_timezone = pytz.utc
    china_timezone = pytz.timezone('Asia/Shanghai')

    # 将日期时间字符串转换为 datetime 对象，并设置为 UTC 时区
    dt_utc = datetime.strptime(date_string, '%Y-%m-%dT%H:%M:%SZ')
    dt_utc = utc_timezone.localize(dt_utc)

    # 转换为中国时区
    dt_china = dt_utc.astimezone(china_timezone)

    # 转换为 UTC 时间戳
    timestamp = int(dt_china.timestamp())
    return timestamp



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2024/8/15 16:27
# @Author  : yh
# @File    : utc2timestamp.py
from datetime import datetime, timezone


def utc2timestamp(utc_time_str, unit="ms"):

    # 解析UTC时间字符串
    utc_time = datetime.strptime(utc_time_str, '%Y-%m-%dT%H:%M:%S')
    # utc_time = datetime.strptime(utc_time_str, '%Y-%m-%dT%H:%M:%S.%fZ')
    # 将UTC时间转换为timestamp
    timestamp = utc_time.replace(tzinfo=timezone.utc).timestamp()
    if unit == "ms":
        return int(timestamp * 1000)
    else:
        return int(timestamp)


if __name__ == '__main__':
    # 示例使用
    utc_time_str = '2023-01-01T12:00:00'
    # utc_time_str = '2023-01-01T12:00:00.000Z'
    timestamp = utc2timestamp(utc_time_str)
    print(timestamp)  # 输出时间戳
    # 1672574400.0



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2024/11/13 8:52
# @Author  : yh
# @File    : current_time.py
from datetime import datetime


def curr_time():
    # 获取当前时间
    now = datetime.now()
    # 格式化当前时间为年月日时分秒
    formatted_time = now.strftime('%Y-%m-%d %H:%M:%S')
    return formatted_time




import setuptools
with open("README.md", "r") as fh:
    long_description = fh.read()
setuptools.setup(
    name="hdsdk",
    version="0.0.9",
    author="wanggf",
    author_email="wanggf@hd.com",
    description="python sdk for hadian",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'pandas',
        'numpy',
        'pymannkendall'
    ],
    python_requires='>=3',
)



import numpy as np
import pandas as pd
import math


def get_var(valLst):
    """
    计算一组数据的方差
    :param valLst: 输入数据
    :return: 输入数据的方差
    """
    result = np.var(valLst)
    if result is not None:
        return float(result)
    return result


def get_std(valLst):
    """
    计算一组数据的标准差
    :param valLst: 输入数据
    :return: 输入数据的标准差
    """
    result = np.std(valLst)
    if result is not None:
        return float(result)
    return result


def get_mean(valLst):
    """
    计算一组数据的均值
    :param valLst: 输入数据
    :return: 输入数据的均值
    """
    result = np.mean(valLst)
    if result is not None:
        return float(result)
    return result


def get_weighted_avg(valLst, weightedLst=None):
    """
    计算一组数据的带权重均值（加权平均值）
    :param valLst: 输入数据
    :param weightedLst: 权值
    :return: 输入数据的加权平均值
    """
    result = np.average(valLst, weights=weightedLst)
    if result is not None:
        return float(result)
    return result


def get_geometric_avg(valLst):
    """
    计算一组数据的几何平均值
    :param valLst: 输入数据
    :return: 输入数据的几何平均值
    """
    total = 1
    for i in valLst:
        total *= i
    return pow(total, 1 / len(valLst))


def get_rms(valLst):
    """
    均方根
    """
    return math.sqrt(sum([x ** 2 for x in valLst]) / len(valLst))


def get_cv(valLst):
    """
    离散系数:通常用来比较两组量纲差异明显的数据的离散程度
    """
    mean = np.mean(valLst)  # 平均值
    std = np.std(valLst, ddof=0)  # 标准差
    cv = std / mean
    return cv


def get_mode(valLst):
    """
    计算一组数据的众数
    :param valLst: 输入数据
    :return: 输入数据的众数
    """

    counts = np.bincount(valLst)
    result = np.argmax(counts)
    if result is not None:
        return float(result)
    return result


def get_median(valLst):
    """
    计算一组数据的中位数
    :param valLst: 输入数据
    :return: 输入数据的中位数
    """

    result = np.median(valLst)
    if result is not None:
        return float(result)
    return result


def get_fgf(valLst):
    """
    方根幅值
    """
    return ((np.mean(np.sqrt(np.abs(valLst))))) ** 2


def get_pp(valLst):
    """
    峰峰值
    """
    result = 0.5 * (np.max(valLst) - np.min(valLst))

    return result


def get_max(valLst):
    """
    最大值
    """
    result = np.max(valLst)
    if result is not None:
        return float(result)

    return result


def get_min(valLst):
    """
    最小值
    """
    result = np.min(valLst)
    if result is not None:
        return float(result)

    return result


def get_range(valLst):
    """
    极差
    """

    return get_max(valLst) - get_min(valLst)


def get_kl(x, y):
    """
    KL散度
    """

    return sum(x * np.log(x / y))


def get_crest(valLst):
    """
    峰值因子
    """
    result = np.max(np.abs(valLst)) / get_rms(valLst)

    return result


def get_clear(valLst):
    """
    裕度因子
    """
    result = np.max(np.abs(valLst)) / get_fgf(valLst)

    return result


def get_shape(valLst):
    """
    波形因子
    """
    result = (len(valLst) * get_rms(valLst)) / (np.sum(np.abs(valLst)))

    return result


def get_imp(valLst):
    """
    脉冲指数
    """
    result = (np.max(np.abs(valLst))) / (np.mean(np.abs(valLst)))

    return result


def get_slope(x, y):
    """
    斜率，numpy内置函数，线性拟合
    """
    slope, _ = np.polyfit(x, y, 1)
    return slope


def get_slope_with_normalization(x, y):
    """
    斜率，先进行线性归一化
    """
    if np.min(x) == np.max(x):
        return 0
    if np.min(y) == np.max(y):
        return -9999

    x_normalized = min_max_normalize(x)
    y_normalized = min_max_normalize(y)

    for val in x_normalized:
        if math.isnan(val):
            return -9999
    for val in y_normalized:
        if math.isnan(val):
            return -9999

    slope, _ = np.polyfit(x_normalized, y_normalized, 1)

    return slope


def get_stability_factor(x, t, corr_pw, corr_wh):
    """
    稳定系数
    :param x: 待计算时间序列数据
    :param corr_pw: 相关系数（有功）
    :param corr_wh: 相关系数（水头）
    :return: 稳定系数
    """
    if corr_wh == -10 and corr_pw != -10:
        if abs(corr_pw) > 0.8:
            corr = corr_pw
            return 1 - abs(corr)
        if abs(corr_pw) <= 0.8:
            slope = get_slope_with_normalization(x, t)
            return 1 - abs(math.tanh(slope))
    else:
        if abs(corr_pw) > 0.8 or abs(corr_wh) > 0.8:
            corr = corr_pw if corr_pw > corr_wh else corr_wh
            return 1 - abs(corr)
        if abs(corr_pw) <= 0.8 and abs(corr_wh) <= 0.8:
            slope = get_slope_with_normalization(x, t)
            return 1 - abs(math.tanh(slope))


def min_max_normalize(x, epsilon=0.001):
    data = np.asarray(x)
    min_val = np.min(data)
    max_val = np.max(data)
    if max_val - min_val == 0:
        max_val += epsilon
    data_normalized = (data - min_val) / (max_val - min_val)
    return data_normalized


def mean_std_normalize(data):
    mean_val = np.mean(data)
    std_val = np.std(data)
    normalized_data = (data - mean_val) / std_val
    return normalized_data


def log_normalize(data):
    max_val = np.max(data)
    normalized_data = np.log(data + 1) - np.log(max_val + 1)
    return normalized_data


def align(x, y):
    df1 = pd.DataFrame({
        'time': x['time'],
        'value': x['value']
    })

    df2 = pd.DataFrame({
        'time': y['time'],
        'value': y['value']
    })

    merge = pd.merge(df1, df2, on='time', how='inner')
    x_val, y_val, t = list(merge['value_x']), list(merge['value_y']), list(merge['time'])
    return x_val, y_val, t


if __name__ == '__main__':
    # 示例数据
    # x = np.array([1, 2, 3, 4, 5])
    # y = np.array([5, 6, 7, 8, 7, 9])

    # data = pd.DataFrame({"数据序列A": x, "数据序列B": y})
    # corrs = data.corr()

    # result = pd.merge_asof(df1, df2, on='time', by='time', direction='forward')
    # x = {
    #     'time': [1, 3, 6],
    #     'value': [10, 20, 40]
    # }
    # y = {
    #     'time': [1, 3, 5, 6],
    #     'value': [5, 10, 15, 20]
    # }
    # x, y, t = align(x, y)
    # print(x)
    # print(y)
    # print(t)

    # try:
    #     a = [1, 1, 1, 1.2, 1]
    #     # b = [0,0,0,3,0]
    #     b = [3.1, 3.3, 3.8, 4.3, 5]
    #     # r = get_slope(a,b)
    #     print(np.max(a))
    #     r = get_slope_with_normalization(a, b)
    #     print(r)
    # except Exception as e:
    #     print(e)
    a = [3.5, 3, 3]
    # b = [0,0,0,3,0]
    b = [0, 0.1, 0.14]
    # r = get_slope(a,b)

    r = get_slope_with_normalization(a, b)
    print(r)
